/* vplaycd.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

// This is a program that uses libcdaudio to play a track from the CD.
// It's something like a very simple version of cdcd, the program that
// libcdaudio ships with, but it is more directly useful with Paloma,
// as it:

// 1) does not terminate until the song is finished

// 2) reports current position in the song during playback, and

// 3) supports playing an arbitrary subset of a given track.

#include "system.h"

#include "cdTrackPlayer.h"
#include "mainProgram.h"

#include <string>
#include <iostream.h>
#include <signal.h>

static const string default_cdrom_device = "/dev/cdrom";

class VPlayCDMain : public MainProgram {
public:
  VPlayCDMain();

  virtual void run();

protected:
  virtual void handle_opts(const string &opt, const string &arg);
  virtual void handle_args(const Args &args);
  virtual void help();
  virtual void describe_options();
  virtual void describe_args();

  static void sigterm_handler(int signo);

  string _cdrom_device;
  double _start_time;
  double _play_time;
  bool _verbose;
  bool _length_only;

  int _track_number;

  static CDTrackPlayer *_player;
};

CDTrackPlayer *VPlayCDMain::_player = NULL;


VPlayCDMain::
VPlayCDMain() : MainProgram("vplaycd") 
{
  define_opt("d", true);
  define_opt("s", true);
  define_opt("p", true);
  define_opt("v", false);
  define_opt("l", false);

  _cdrom_device = default_cdrom_device;
  _start_time = 0.0;
  _play_time = -1.0;
  _verbose = false;
  _length_only = false;
}

void VPlayCDMain::
run() {
  _player = new CDTrackPlayer(_cdrom_device, _track_number);
  if (!_player->is_valid()) {
    exit(1);
  }

  // Install a signal handler on SIGTERM and SIGINT and anything else
  // likely, so we can stop the music prematurely if we're terminated.
  struct sigaction act;
  act.sa_handler = sigterm_handler;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;
  if (sigaction(SIGTERM, &act, NULL) < 0) {
    perror("sigaction");
    exit(1);
  }
  if (sigaction(SIGINT, &act, NULL) < 0) {
    perror("sigaction");
    exit(1);
  }
  if (sigaction(SIGHUP, &act, NULL) < 0) {
    perror("sigaction");
    exit(1);
  }

  if (_length_only) {
    cerr << "Track " << _track_number << " is " 
	 << _player->get_length() << " seconds.\n";
    exit(0);
  }

  if (_start_time > 0.0) {
    if (_play_time >= 0.0) {
      cerr << "Playing sub-range " << _start_time << " to " 
	   << _start_time + _play_time << "\n";
    } else {
      cerr << "Starting playback at time " << _start_time << "\n";
    }
  } else if (_play_time >= 0.0) {
    cerr << "Playing only the first " << _play_time << " seconds.\n";
  }
  
  _player->play(_start_time, _play_time);
  
  while (_player->is_playing()) {
    if (_verbose) {
      cerr << "time " << _player->get_current_time() << "   \r";
    }
  }

  cerr << "\nDone.\n";
}

void VPlayCDMain::
handle_opts(const string &opt, const string &arg) {
  if (opt == "d") {
    _cdrom_device = arg;

  } else if (opt == "s") {
    _start_time = atof(arg.c_str());
  
  } else if (opt == "p") {
    _play_time = atof(arg.c_str());

  } else if (opt == "v") {
    _verbose = true;

  } else if (opt == "l") {
    _length_only = true;

  } else {
    MainProgram::handle_opts(opt, arg);
  }
}

void VPlayCDMain::
handle_args(const Args &args) {
  if (args.size() != 1) {
    usage();
    exit(1);
  }

  _track_number = atoi(args[0].c_str());
}

void VPlayCDMain::
help() {
  cerr << 
    "\n"
    "This program plays audio tracks from a CD-ROM drive.  It is somewhat\n"
    "specialized to enable it to work well with Paloma.\n";
  MainProgram::help();
}

void VPlayCDMain::
describe_options() {
  cerr <<
    "  -d device  The cdrom device to access.  The default if this is unspecified\n"
    "             is " << default_cdrom_device << ".\n"
    "  -s secs    Start at the indicated number of seconds into the track.\n"
    "  -p secs    Play for the indicated number of seconds, and stop.\n"
    "  -v         Output an indication of progress through the track.\n"
    "  -l         Report only the length of the track in seconds, and terminate.\n"

    "\n";
}

void VPlayCDMain::
describe_args() {
  cerr << " track_num\n\n";
}

void VPlayCDMain::
sigterm_handler(int signo) {
  if (_player != NULL) {
    _player->stop();
  }

  exit(1);
}


int
main(int argc, char *argv[]) {
  VPlayCDMain m;
  m.command_line(argc, argv);
  m.init();
  m.run();
  return (0);
}
